#include "global.h"
#include <util/delay.h>
#include <avr/io.h>

#include "uart.h"
#include "rprintf.h"

#define N 10

// defines
#define ASCII_BEL				0x07
#define ASCII_BS				0x08
#define ASCII_CR				0x0D
#define ASCII_LF				0x0A
#define ASCII_ESC				0x1B
#define ASCII_DEL				0x7F


void initRS232() {
	uartInit();
	uartSetBaudRate(_baudr);
	rprintfInit(uartSendByte);
}

int main(void) {
	DDRC = 0xff;
	int i;
	u08 c;

	initRS232();
	
	for (;;) {
//		rprintf("i = %d\r\n",i++);
		
		while(uartReceiveByte(&c)) {
			if( (c >= 0x20) && (c < 0x7F) )
			{
				uartSendByte(c);
			}
			else
			if(c == ASCII_CR)
			{
				uartSendByte(ASCII_CR);
				uartSendByte(ASCII_LF);
			}
		}
		
		PORTC = ~(1<<5);
		PORTC = ~(1<<6);
		PORTC = ~(1<<1);
		PORTC = ~(1<<0);
		PORTC = ~(1<<2);
	}
	
	return 0;
}
